# RPM specfile for the MicroEmacs package
# Copyright 2008, 2017, 2020 Parliament Hill Computers Ltd, all rights reserved.
# Author: ADDW December 2008
# SCCS: @(#)	4.40 07/19/22 10:17:57

# BEWARE: MicroEmacs.spec is built from MicroEmacs.spec.template

%define name MicroEmacs
# Define the version number, etc.
# These come from evers.h, but that will not be usable very early on
# when it is in a tar file. Thus this .spec file is built from a template.
# Beware: several things start VERSION:
%define version 4.21
%define release 0.0

Summary: A version of the Emacs text editor that runs in a terminal
Name: %{name}
Version: %{version}
Release: %{release}
Source: http://www.phcomp.co.uk/Downloads/MicroEmacs-%{version}-%{release}.tar.gz
Vendor: %{vendor}
URL: %{phcl_url}

License: Public Domain

Packager: %{packager}

Group: Applications/Editors
BuildRoot: %{_builddir}/%{name}-root

# Documentation directory.
# This changed with Fedora 30 (or something) and RedHat/CentOS 8
# It used to be /usr/share/doc/package-version, went to /usr/share/doc/package
%if 0%{?fedora} >= 30 || 0%{?rhel} >= 8
%define docd %{_docdir}/%{name}
%else
%define docd %{_docdir}/%{name}-%{version}
%endif

# Need this to avoid error about empty files: xxx/debugsourcefiles.list
%global debug_package %{nil}

# Oct 2020 phcl-MicroEmacs (4.20) was renamed MicroEmacs
Provides: phcl-%{name} = 4.20
Obsoletes: phcl-%{name} < 4.20

# Cope with package name change
%if 0%{?rhel} >= 8 || 0%{?fedora} >= 30
%define	perlPackage perl-interpreter
%else
%define	perlPackage perl
%endif

%if 0%{?fedora} >= 6
Requires: coreutils
Requires: ncurses-libs
Requires: ncurses-base
Requires: gzip

BuildRequires: make
BuildRequires: sed
BuildRequires: gcc
BuildRequires: autoconf
BuildRequires: ncurses-devel
BuildRequires: %{perlPackage}
%else
%if 0%{?rhel} >= 6
Requires: coreutils
Requires: ncurses-libs
Requires: ncurses-base
Requires: gzip

BuildRequires: make
BuildRequires: sed
BuildRequires: gcc
BuildRequires: autoconf
BuildRequires: ncurses-devel
BuildRequires: %{perlPackage}
%else
Requires: coreutils
Requires: ncurses
Requires: gzip

BuildRequires: make
BuildRequires: sed
BuildRequires: gcc
BuildRequires: autoconf
BuildRequires: ncurses-devel
BuildRequires: %{perlPackage}

Requires: coreutils
Requires: ncurses
Requires: gzip
%endif
%endif

# This is only needed if we want to build a relocatable RPM:
#Prefix: %%{_prefix}

# The description below is also in README.intro
%description
A text editor in the Emacs family: small, light. Its screen is in a terminal
(or emulator - eg Gnome, Xterm), not a GUI, so performs well over ssh. Handles
Unicode characters, UTF-8 encoding, eg: £ € ∞ Ω γ チ → ä ½ é ש ب ک ࿋ ✔ 𝅘𝅥𝅮

* Edit multiple files simultaneously
* Split the Emacs screen horizontally and vertically into Emacs windows each showing different buffers/files.
* Good documentation, help subsystem: keyword search, topic hyperlinks
* Tutorial for new users
* window-menu command that makes window operations easy:
  split, join together, move the 4 edges, kill/close, change buffer in window, move cursor to different window
* UTF-8 characters displayed properly (including double width; zero width shown as a position)
* Can insert Unicode points as hex/decimal/octal if the keyboard cannot generate it.
* Window options to display: tabs in colour, line endings, line numbers, cursor position (column/line number)
* Can track cursor between windows: same up/down in tracked windows
* Auto indenting for C and PHP, region indent/unindent, region un/comment
* Handles binary files, arbitrary line lengths.
* Hexadecimal edit mode. At the same time in another Emacs window buffer can be viewed as text when editing hex (or view hex when edit text)
* Editing of prompts for command & file names. Can insert current filename/word-under-cursor/...
  Tab does file name expansion.
* User can place alphabetic marks, Emacs places numeric marks with some buffer position jumps
* Position of mark shown in red, in non current window "." in green
* Meta (top bit set, non UTF-8) & control characters shown as one underline/standout character, option for old behaviour (eg M-^C)
  This preserves alignment of columns.
* Last char of a over long line shown in yellow; move the cursor to it & whole screen scrolls left/right not just one line
* Can change foreground/background colours.
* list-buffers also shows much Emacs info: variables, window attributes, statistics
* Search: a space can match multiple space, tab, newlines
* Search/replace intelligently handles capitalisation & line breaks in replaced strings. Easy to have several replacements
* DOS buffer mode - auto handle MS Windows line endings
* RENEW buffer mode - like: tail -f
* Macro/programmable

Emacs has a help system, get there by pressing Escape ?

A hacked version of MicroEmacs 3.7 to 3.9. It comes with some own key bindings, many bugs fixed.
Version %{version}

This should work on a variety of operating systems. This fork
has only, recently, been tested on: Linux; FreeBSD; Minix

http://en.wikipedia.org/wiki/MicroEMACS
Written by: Dave Conroy and Daniel Lawrence - so original copyright belongs to them.
Many hacks by Alain Williams - released under the GPL version 2 or later.
%prep
%setup -q

%build
# Need -g else get complaint on missing debug info
make CFLAGS='-g -std=c99 -O2 -DLINUX5 -DDOCDIR=\"%{docd}/\" -DDOC_DIR=\"%{docd}/docs/\" -DPREFIX=\"%{_prefix}\" -DSYSCONFDIR=\"%{_sysconfdir}\"' VERSION=%{version} RELEASE=%{release} TERMDRIVER=tcap.o LIBS=-lcurses


%install
echo DOING INSTALL
rm -rf $RPM_BUILD_ROOT
make DESTDIR=$RPM_BUILD_ROOT%{_prefix} VERSION=%{version} RELEASE=%{release} DOCDIR=$RPM_BUILD_ROOT/%{docd}/ SYSCONFDIR=%{buildroot}%{_sysconfdir} install
echo DONE INSTALL

%clean
rm -rf $RPM_BUILD_ROOT

%files
# Make file mode read only - reduce attack surface
# Do not make install in the Makefille have '-m 444' else get cannot write errors :-(
%defattr(444, root, root, 555)

%attr(755, root, root) %{_bindir}/me

# It would be nice to do the following, but, for some reason, rpmbuild winges: File listed twice
# %attr(755, root, root) %%{docd}/scripts/GenHelpIndices.pl

%doc README INSTALL LICENSE VERSION_RELEASE Fixes.gz Hacks.gz emacs.rc DotEmacsrc docs/ TestFiles/ help/ macros/ scripts/


%config(noreplace) %{_sysconfdir}/emacs.rc
%{_mandir}/man*/microemacs.1.gz

%changelog
* Tue Jul 18 2022 Alain Williams <addw@phcomp.co.uk>
- 4.21
- Help & other files gzipped on install. Where they are placed rationalised
- New $append-emacssave
- New functions &gmtime &localtime. Use variables %_Weekdays[] & %_Months[]
- Cope better in out of memory situations
- Implement &join, make &indirect work
- Implement new command: static -- variables in procedures
- New $open-url - Command to use to show a URL in a web browser.
- New $wm-separator - separates arguments to write-message
- write-message fix bugs when: &call, when no argument, duplicates in [Message History]
- return-alpha-mark fix bugs: status returned, query marks is not an error
- apropos puts "proc" after procedures
- Implement command line syntax: @+files @-files - late/early running command line macros; chage @files
- New $startup-file-done
- Fix &lower &upper to work correctly if the string contains a NUL
- Implement -P command line option -- helps gdb in early running code
- incremental-search, query-replace-string: move-window-left & move-window-right keys: now work after scroll past window edges
- Reduce scope of some declared variables
- Output NL after execute-program & pipe-to-command to not lose command last line if not followed by a NL
- Delay effect of -r (restrict) cmd line option until after startup files are run
- Command line option +-10 (from bottom) line number now works
- incremental-search puts end marker at end of prompt so trailing space is visible
- incremental-search emptying the search string does not end search, return to start
- Comments by file suffix looked up in %_SuffixInfo - set in start up file
- $blank-comment controls how blank lines are commented
- incremental-search - use of previous search string made more natual (can edit ...)
- Prompting, add C-x s (current search string), C-x v (variable value)
- RENEW mode no longer tramples on what it shown in the message line, including prompts
- RENEW mode suppress some Opening/Closing messages - not flood message buffer
- Terminal: when inserting characters ensure not in standout mode
- split-horizontally-and-find-file & split-and-find-file not leave split window when cannot open the file
- unmark-buffer now looks a the count. Can be used to test if buffer modified
- renew-buffer was using the wrong history buffer
- Search default to not-word-match. This was wrong for search done due to -s command line option
- widen now looks at its argument and can be used to test if a buffer is narrowed
- TRUE FALSE now macro tokens
- Fix bug of search options being wrongly copied between lines in search history
- Mode RENEW (append) big CPU saving on large files
- Add command line option -G, -g can also be used to set global modes
- Fix bug in what add-mode, etc, showed in response to ? or ??
- add-mode, add-global-mode: use cycle-background-colour/cycle-foreground-colour to do the work. Better protection
- Add SYSTEM to the list of colours understood
- Permit breaking array/hash literal values over several lines
- Zap new bug when dobuf() called recursively
- Fix crash in macro when invalid code follows @ or #
- Add macro function &values
- Interactive set command will assign values that are array, hash, bool, ...
- Test that literal arrays & hashes end with ] or }
- Fix bug where low alpha sorting procedure added to proc binding table. Add procs to --list-bindings output
- New function &same: true when args are the same variable
- Move test for buffer executing and modify commands earlier
- On recursive call of procedure the context of !while is saved/restored - fix bug
- Fix local/global created vars only taking strings when created
- Executing a file no longer checks for .emacssave file
- Add bind of procedures look-word next-num-mark eval-expression
- New command load-procedure
- static variables fix scope & lifetimes: scope is procedure and buffer. local tweaks
- global static local commands can only be executed in a macro.
- Non executing procedure buffers can now be killed
- Check correct nesting of !if and !while
- Fix state on error parsing procedure
- Fix line count could be wrong in a narrow
- !force now makes failing procedures work
- &in/&sin now do range checks when a value is an array. Unify common &in/&sin code
- New $in-index $in-key $in-range
- On buffer switch/restore with window preserve the leftoffset
- Fix bug delete-previous-word not deleting first character in a buffer/narrow
- fill-paragraph only complains about end of buffer if that is where it starts
- Fix bug in magic query-replace-string where .* stopped one character short
- previous-paragraph & next-paragraph now only return TRUE if moved all count paragraphs
- Check that ||http://URL ends in a space
- Add memory corruption checks
- write-message checks if an argument is in error
- write-message now prints the elements of an array if passed as the argument
- ^Z while entering command line (eg prompt for file name) does suspend to shell
- $status after running a procedure is that of the last command run
* Tue Oct 06 2020 Alain Williams <addw@phcomp.co.uk>
- 4.20
- newline - will trim before the inserted NL if in WRAP mode
- Try & get WRAP mode & fill-paragraph do exactly the same thing
- Change prompt handling: some interactive strings can be very long
- New buffers: [Help Search] & [Buffer History]
- reframe() - do full reframe when "." at top of empty window & there are lines above
- Max file path length is now 4096
- change-directory deduces relative path names where it can
- write-message - no longer puts the message to [Message History] twice
- New variables $term-opened $early-log $eof $cmd-flag & $cmd-count
- Emacs can now work in a pipeline
- Fix bugs when creating procedures. More error messages
- execute-macro-1 to 40 and execute-procedure no longer have a default argument, accept -ve argument
- New variable $max-question. mlyesno() notices EOF on stdin and looks at $max-question
- Changes to internal structures to extract a TEXT (string) structure from a LINE
- Fix pipe-command that was putting 2>&1 on end of command in history buffer
- Fix bug with procedure !return being assigned to the wrong variable
- Stop core dump on inputting encryption key, also blat history buffer after key entered
- Correctly read multibyte characters when the terminal has not been fully initialised
- Fix prefix bug when finding files - help, etc.
- Make &gtk the longer name &gtkey
- goto-matching-fence noticed C++ comments when not in CMODE
- A -ve count to reyank now pastes more recently deleted text
- Name format checks on creating user variables.
- Numbers can be prefixed with '+' (as well as '-')
- New $pwd variable
- Prompts: add C-x f (last component of current file path), C-x d (current directory)
- Make search string set with -s command line, the default for the next search
- Fix bug window not l/r scrolling in query-replace-string
- Window left/right scroll is now aware of line numbers
- Say what is wrong when search history runs out
- Procedures: bindable to keys (new command bind-procedure-to-key), auto-loaded if not known
- Fix prob with file called "main" on OS command line
- User vars can be typed: str, int, bool, char, array, hash. Vars assigned by reference counting
- global & local commands will prompt for optional value to variable
- &docmd &count &keys &push &pop &shift &unshift &stat !continue '!while var array'
- Tidy prompting, eliminate 2 of triplicate copies of code
- Got rid of fixed sized buffers in command prompts when run in macros.
- Procedure formal arguments can be arrays - so procedures use variable numbers of arguments
- Procedure arguments and return can be typed. Eg: receive array/hash !return array/hash
* Sun Jul 05 2020 Alain Williams <addw@phcomp.co.uk>
- 4.19.1.1
- Fix permissions problems when building RPMs
* Fri Jul 03 2020 Alain Williams <addw@phcomp.co.uk>
- 4.19
- query-replace-string implement 's' command - show last change
- goto-line - numbers < 0 means from the bottom of the buffer
- goto-column now takes -ve numbers to mean from the end of the line, as does command line +-nn.-cc
- append-file no longer writes to associated file by default. Prompts for a file name
- New commands: narrow-to-region & widen-from-narrow (synonym widen)
- Increase size of binding table from 256 to 280
- pipe-region-to-command removed, pipe-buffer-to-command renamed pipe-to-command which can be narrowed
- write-region: was getting the last line wrong; no longer sets buffer's file name, modification status
- Fix help bug caused by first line of a help file being a comment
- shell-command when run in a keyboard macro prompts the user and pauses before continuing, unless count is 0
- Fix bug where file named like 123456789012345.conf was put into C mode as buffer name is 123456789012345.c
- Renew mode: when setting & when checking: now shows message when a buffer is renewed
- Separate use of bottom line as message line & prompt line
- New variable $enable-flow-control; C-s & C-q can be used; C-s bound to incremental-search
- Add clear screen on entering visual mode, TI does not always do this, eg FreeBSD
- command buffer, mode line, remove misleading 'File:'
- pipe-command. Fix bug when modified buffer resulted in prompt attempt when not in visual mode
- Tidy/simplify what can write to the message/prompt line & buffer usage
- 2 more numeric marks - only internal use. Fix buffer RENEW crash while in incremental-search
- kill-to-end-of-line now works with -ve count - as it should have done
- Fix yank adding an extra line at the end of a narrow
- trim-buffer now leaves "." & top-of-page at the end away from where lines were trimmed.
- When RENEW on a buffer assumes append - the mode is shown in lower case: mode line & list-buffers
- tags: Does not produce spurious message if the user types C-g
- Installation: documentation files, etc, mode 644 -> 444
- Some commands now do not work on an executing buffer
- Return values fixed for next/previous-line move-window-up/down - hit top/end messages generated
- incremental-search & query-replace-string act nicely with move-window-up/down making the search point visible again
- incremental-search - replay buffer no longer fixed size
- Fix help issues
- DOS mode can be set in a VIEW mode buffer, it isn't really a change
- incremental-search set options with C-o : case independence, what SPC matches; not adjacent to word
- filter-buffer no longer does a widen, will filter a narrow if one is in effect
- Fix bug in history buffer adding/trimming when the buffer is displayed
- Do a better job of copying numeric marks on trim-buffer
- New [Trace Buffer] - compile time option to help find rarely seen bugs
- Fix prob in search that could result in "." being out of window (left/right)
- query-replace-string 'o' to prompt lets setting of search options
- When a buffer is auto-RENEWed window's mode line cycles C(hange) flag: r R ᴙ Я
- Fix bug where message line sometimes shown empty
- incremental-search prompt only shows last search string while no new one has been entered/started
- Determining comment (eg comment-region) now skips blank lines at start of file and spans white space at the start of lines
- Improve message when writing to a file - particulary on error
- Not prompt user when writing to a file that had been removed
* Wed Apr 22 2020 Alain Williams <addw@phcomp.co.uk>
- 4.17
- ctag search: fix bug where a space tried to match more than it should
- --version option now has date of version & compile time
- DotEmacsrc: More comments, including ones starting # Examples of $suffixes-c & $suffixes-php
- Help: Allow **b:;nn** in .help files to get padding
- New variable $file-symlink - last opened file was a symbolic link
- New variable $file-numlinks - number of links on file just opened for read/write
- comment-region no longer looks as PHPMODE - which is used for Perl scripts
- Fix insert-tab so that in C mode spaces after a comment continuation are converted to TABs
- Fix redraw-display so that with no argument if refreshes the last page with "." on the middle line. This was broken by improvements to reframe()
- When showing bound keys, change display of char after a prefix. Eg C-x RET -> C-x C-m
- On file readin when setuid root, if no write mode bit is set mark the buffer read only
- Setting $sres now works
- save-file now offers to remove a .emacssave file if the buffer was read from it
- Do not make buffer VIEW if the file does not exists but we read a .emacssave file
- save-file checks if there is a new .emacssave file; save-many-buffers will not try to write invisible buffers
- Duplicate file detection (on open) - says if link or symlink (not perfectly)
- Fix bugs that cleared numeric marks
- Add internal documentation
- Fix problems with Minix port
- Avoid possible race condition with SIGWINCH
- query-replace-string now understands up/down arrows to select replacement history
* Mon Mar 16 2020 Alain Williams <addw@phcomp.co.uk>
- 4.15
- Ensure that README is rebuilt
- query-replace-string implement 'b' - search backwards
- query-replace-string on abort (C-g) say how many subs done & not needed
- When reading a file, report if it has more than 1 link
- query-replace-string 'p' -> 'E', '^' synonym for 'r' - keys that GNU Emacs uses
- query-replace-string 'q' synonym for 'C-g'. Help via 'm' or '?' is now 3 lines
- C-l can now be used to refresh the screen in many places, eg: search, prompts
- Help: search: ^ means exact on 1st word only and must match, new line type c in _keywords file
- Help: key words after commands
- Apropos buffer - accept Emacs commands that have digits in their name
- apropos now shows bindings on names of both commands of synonyms
- $fillcol minimum now 0, was 10 - consistent with elsewhere
- execute-program becomes synonym for shell-command -- was almost the same thing
- Help: Stop unwanted title lines appearing in search results
- Can run macros from the command line
- name-buffer: when given buffer name that already exists - complains, not prompt again
- Implement goto-column bound to M-G
- Command line -g nn.cc - cc is now column positions (as other editors) not byte offset
- $curcol counts double width chars as width 2
- New variable $curcharpos
- Old window could keep ornamentation when [Help] made current again
- For consistency: column numbers shown/settable count from 0 - no longer 1, sometimes
- Fix calculation of top line number sometimes going wrong, in several places
- list-buffers move Mode right 1 place in case one file is setuid (needing extra character)
- Mode check not clearing mode on error. Bad UTF-8 was disasterous
- goto-line allows line # one beyond end of buffer
- Fix # lines in buffer not being maintained: filter-buffer & char insert at very end of buffer
- Check required string lengths on prompts
- Set minimum lengths: $cbufname $utf8-replacement $TERM $language
- Add 'not implemented' messages for window (ie screen) size change
- More validation when setting environment variables
- Screen paint: work 2 lines at a time when work out from line containing "."
- Fix bug where cmd line -v could be applied to more files than it should be
- Global modes only applied to buffer when it comes active
- Check C/PHP mode not done on inactive buffer, looks at buffer name if no file name
- Implement command line option +0, only apply to next file
- A removed file is marked 'x' not 'X' on mode line & list-buffers
- New variables $suffixes-c $suffixes-php - enable choice of suffixes
* Fri Jan 10 2020 Alain Williams <addw@phcomp.co.uk>
- 4.14
- Port to CentOS 8
- Fix cursor position bug command in line prompts, so M-b, etc, did not work
- Comments in macro files can start '#' as well as ';'
- Command prefixes (M- & C-x) & command line 'Quote: ' after C-v are highlightened when displayed after $prefdlay
- Make prefix characters lower case (C-x C-u)
- If command line @file-name contains a '.' or '/' search path, suffix is not done
- Use terminfo capability 'it' (init_tabs) rather than nail it to be 8
- move-window-left/right works out leftoffset
- Fix screen corruption bugs when cursor goes to last column of the screen+; also on Xterm;
- append-file messages now have the word 'append'
- Ensure that move-window-* does not upset how search works
- New command show-marks, default bound to C-x M
- return-alpha-mark '+' & '-' understood as next/previous numeric mark
- Fix numeric mark bugs
- list-buffers to show values of path values if there is an env var
- Fix bugs with s/replace undo: next subsequent match missed; not undo enough where SPC match several white space
- Preserve framing (if possible) on redraw when the screen is garbage
- Restore of alpha/num marks on last line in buffer now works
- next-page: fix bug where "." after last line could be placed at not start of line
- Accept/ignore flag '@' in searches - meaningful to GenHelpIndices.pl
- Help: Insist that there is not a space/tab after start || of a link
- Help index generation checks that a place exists for each link
* Tue Nov 19 2019 Alain Williams <addw@phcomp.co.uk>
- Documentation directory no longer has version number (Fedora >= 30 && rehdat >= 8)
- Tar file has release as part of its name
* Sun Nov 17 2019 Alain Williams <addw@phcomp.co.uk>
- 4.13.1.3
- Add 'Help' to description of describe-bindings & describe-key
- Separate Opening from Writing messages - in case open takes a long time.
- Shorten messages. Do fflush() before syncing message
- Remove screen redraw from splitwindhoriz() - not needed, generated much output. + consequential fix
- Vertical window edges updated en masse - avoid many pen changes
- Use terminal's ability to insert/delete characters on a line. New $scroll-lr
- Can elide operating system command line options. Eg: -Nv
- forward-character-several (& back) works in terms of position, not character (think tabs, double width characters)
- Hardware scroll left/right: won't delete white space at line end - not needed
- Various screen redraw tweaks/optimisations
- Fix bug where backspace in isearch was not recognised as the delete previous command
- New statistic: number of Emacs commands executed
- Fix bug on restore "."/marks after buffer reload
- New command: write-region
- Fix buffer read overrun in reporting of UTF-8 error on file read
- Enable non zero left offset in the message window. Also cures display buffer overrun bug.
- Screen size changes: fix race condition on size & allocated buffer changes
- incremental-search & query-replace-string will allow adjustment of the view of the window onto the buffer, move-window-right, etc
- find-tag the found line will be at 1/3 from the top of the window
- Fix bug in goto-matching-fence & when a ')' is entered following unmatched quote
- Test/ added to make, not much there yet - Emacs tests here
- Finging files for macros mainly don't do path search if there is a / in the name
- write-message - the message is flushed to the message line
- Messages to message line before full terminal start are written to stdout - for use in OS command line @macros
* Thu Sep 26 2019 Alain Williams <addw@phcomp.co.uk>
- 4.13.1.2
- Fix display errors
* Thu Sep 19 2019 Alain Williams <addw@phcomp.co.uk>
- 4.13.1.1
- New command: split-horizontally-and-find-buffer, bound to C-3 replacing split-window-horizontally
- Binding for pipe-buffer/region-to-command now built in, not needed in startup files
- New vars: $scroll, $jump, $jump-no-scroll. Synonyms: $tab for $tabwidth, $overlap for $page-context
- reframe will not an empty window after end-of-file, fill most of window
- move-window-down/up puts "." on top/bottom of window, not middle of window
- filter-buffer: Fix: fcntl/F_SETFD - bad args, SCCS expand error, buffer overrun in writev() vector
- 64 bit file sizes for RENEW mode
- New commands: detab-region entab-region trim-region display
- Fix hashing bug, improve/speed-up strings hashed
- Move some bindings from startup file into default bind table
- Rework next-page & previous-page so that "." moves down/up an exact number of pages
- Fix bug where lines appended to history buffer were not shown as "." on last line
- Use memmove() for memocpy() and memrcpy(), use memset() in one case
- New macro function &number
- Fix bug in insert-string - it was putting a newline into an empty buffer
- Command "nop" returns TRUE (was FALSE) as do other Emacs. If count is set: returns that count
- $status is set to what the last Emacs command returns
- Fix macro memory leaks; comparison of too short string
- Fix: use of out of bounds stack array: &upper &lower &xlate &trim; broken debugging
- Avoid division by zero by &divide and &modulo
- Macro functions: number of arguments verified, those that should be numeric checked as numeric
- Terminal movement: more optimisations
- Fix bug generating NUL on message line
- Fix bug move-window-down/move-window-up not maintaining column target
- In hex window option: leftoffset constrained to be multiple of 3 'xx ' & related fixes
- Command line not always done right, eg second command after shell-command
- More -K debug levels
* Tue Aug 20 2019 Alain Williams <addw@phcomp.co.uk>
- 4.13
- Fix warnings on ports to FreeBSD, Minix.
- Check that VERSION, etc, are properly defined in Makefile
- Tidy up rval value, which is now 256 on unknown, not 255
- Guessed single-line comment string taken from first line in the buffer, used in goto-matching-fence & others
- Rework where test files & similar are installed; other Makefile stuff
- list-buffers fix bug where window edges were not shown correctly, sometimes
- goto-matching-fence hacked again, slight improvements
- Improve what happens CMODE and RET is typed
- Implement buffer specific start-of-line comments
- fill-paragraph Fix bug where non word at line end was moved to start of the next line
- Fix bug in next-paragraph previous-paragraph in C/PHP/INDENT where 'empty' comment lines
  were not properly skipped over. Knock on bug caused in fill-paragraph
- Fix bugs where line # not always updated if buffer displayed in multiple windows
- Add const qualifiers to function arguments
- Help keywords now case insensitive. Fix bug where &= flags were sometimes ignored.
- More Out Of Memory checks
- Remove associated help file name - so renew-buffer will refuse to act on [Help]
- Documentation/Help more written
- .spec: Perl required for build.
- Create test build. Tests slowly added.
- Environment variable $term renamed $TERM - as per shell environment.
- apropos, describe-bindings: show command synonyms
- Fix new bug of not matching (), etc, if "." started in a string. Fix infinite loop. Fix prob with quote in comment
- Fence (eg goto-matching-fence) shows an error if the fence is not found
- Fix bug in pipe-command: make command buffer read only, not current window
- Fix bug in file expansion of symbolic links and ..
- comment-region to do a bit more to recognise PHP code
- Fix bug where terminal key names were returned as incorrect - when some were not defined in terminfo
- Fix bug next-paragraph/previous-paragraph sometimes wrapped when hit end of buffer (in indent/wrap mode)
- New option -- ends -options, +nn, @macros
- Put R on mode line if a buffer has been RENEWed
- Check if buffer needs renewing immediately ofter setting RENEW mode
- Include some design notes in the distribution tar file
- Ignore directive lines in help files
- Check if buffer needs renewing immediately ofter setting RENEW mode
- On set/unset RENEW mode forget if the buffer has been auto-renewed
- ffclose() & pi_close() preserve the value of errno.
- Report previously silent errors related to pipe-command
- Help: ||>s: made a synonym for ||>search: Help keyword prompt no longer forced to lower case
- Help: If a key search finds exactly 1 result: display it, don't show a select results page
- Search: A single space matches one or more: SPC, TAB, NL
- New $space-match-white, $search-replace-same
- Search/Replace: better match case of matched string. Handle newlines in replacement.
  Do nothing if matched == replacement. Test $search-replace-same
- C/PHP buffer modes: fix bug tab width was wrong in PHP mode
- On macro error make it easier to see [Message History]
- shell-command when running in a macro only runs the command once
- Implement backward-character-several forward-character-several next-line-several previous-line-several
  Change default bindings for move-to-window-down move-to-window-up move-to-window-left move-to-window-right to make place
- Macro debug: fix bugs, auto call list-buffers if [List] is visible, tweak prompts
- Fix problems with #buffer-name
- Default place of "." in history buffer now after last line
- Number arithmetic now done as long
- File opens logged to [Message History]
- New commands: pipe-buffer-to-command pipe-region-to-command
- Count to emacs commands is now type long
- Handling of child processes improved & done more uniformly
- Fix format string error
- Search: EXACT mode should affect space matching white space
- Shorten sub shell messages
- New command test-emacs but only compiled in if TEST_HOOK
- Implement hashes, speed up: keyboard key -> command; env vars; emacs commands. More stats
- Change ASAVE in mode line from '+' to 's'
- Fix infinite loop bug if first command line file has a .emacssave
* Mon Mar 25 2019 Alain Williams <addw@phcomp.co.uk>
- 4.12
- New help mechanism in place. Help files not complete but good coverage
- describe-key now shows help page for command if given an argument
- New command comment-region bound to M-C
- New commands: split-window (splits H or V - what is best), split-window-vertically. Remove split-horizontally. New $pref-col-horiz gives 'best' width
- New default bindings: move-window-right FN16, move-window-left FN15
- New commands move-window-several-down, move-window-several-left, move-window-several-right, move-window-several-up
- New $notice-emacssave env var. If true auto view mode on files with .emacssave suffix
- New variable $utf8-error
- New --list-bindings command line option
- New $bell-on-renew - Ring the terminal bell on auto renew if TRUE
- New $nobell that stops Emacs ringing the terminal's bell - keep quiet
- New $language & $territory initialised from $LANG
- New b_toplinenum. Fix bug with bad line numbering on return to a window
- Now recognises line numbers (to go to) in tags file; good for #define, etc
- TAB and RET may now be used in bind-to-key command, also used to describe bindings
- Numeric marks will be restored when a window redisplays a buffer
- findFile(), new ways of searching paths, eg used by startup
- Buffers can be marked immutable, cannot unset VIEW mode
- If a .emacssave file is read in place of another file: mark the buffer modified, mode line shows 'S'
- Rework writing to emergency save file, use file name then buffer name, directory path changed by %_path-save -- if set
- Add to explanation in list-buffers
- Add recognised C mode keywords: until, unless, elseif, elseif, do
- pipe-command shows name of the command read from
- Add M-l to prompt editing, lower/down case a word. This should have been there anyway!
- list-buffers distinguishes between file not existing and parent directory not existing
- list-buffers checks file inode status change
- list-buffers Mode flags produced in reverse order so that the contents of $gmode make sense
- list-buffers Only show file type D if it exists on the operating system
- move-window-left no longer tries to keep "." $window-move-margin from the edge
- Adding history lines can now add lines with NUL bytes
- Fix long standing spelling mistake. $od -> $os
- Fix bug with HEX option in insert mode on lower nyble
- [Message History] has 10 times more lines saved than other history buffers
- Fix incremental search history - as it should have been
- Fix issue where command line file name like /etc/ results in empty buffer name.
- Rejig bindings of move-window-* so that the same function keys are reused - Xterm anyway
- DotEmacsrc: Use C- notation, no longer ^ for controls. Play with bindings for move-window-*up/down/... nicer. Move colours to other FN keys
- Redo filename expansion so that it copes with filename containing newline & shell specials like: ;<&
- File name expansion: now works if directory name contains a space
- Command descriptions tweaked
- Rework prefix handling, simplify with a couple of global variables
- Max kill buffer entries was NKILL (15), now $message-history
- $rval on Unix now holds child process exit code
- $term env var is writable. Start up files now done before term init so they can change $term
- Big improvements in CMODE: auto formatting, insert }, etc.
- Allow newline to be inserted into a line in an invisible buffer by insert-unicode & quote-character
- split-window will split horizontally or vertically depending on what looks best
- Fix $lwidth so that it shows the display width of a line, ie understands TABs & UniCode chars
- Rework how file inode info is got/checked, better detection of buffer ageing after shell out, etc
- On writing a file: better checks if we are overwriting something that is old
- find length of longest command name - don't guess
- execute-named-command when interactive & given empty command name no longer gives an error message
- insert-command & insert-file reworked to insert at "." not before the current line
- Fix bug with next-paragraph and line starting '.'.
- Fix Makefile dependency for version number & get from edef.h, not spec file
- Change the way that the RPM .spec file is built.
- Internal: t_mrow & t_nrow are now the number of screen lines, not screen lines -1
- Internal: compile on Minix & small workaround for clang bad code generation
- Improvements on how environment variables are got/set
- kill-buffer if read file in message is about that, not the buffer killed
- Fix bug: window borders not always properly painted when a window was killed
- Deleting newline checks if alpha/numeric marks need removing
- Fix bug of '[Prompt Line]' buffer being marked as changed, confusing to the user
- Characters now inserted via self-insert command -- removes need to handle differently
- Fix bug when newline typed in an empty buffer - nothing happened
- Report if some command line files are duplicates
- Die() does not try to output last few editor commands or message history - if there is nothing there
- Prevent (unlikely) double free()
- Fix bug in ldeln() - just used for the first time.
- Better way of connecting input file to a pipe that we read.
- Pipe open: On fork error ensure that pipe FDs are closed. Child will exit code 127 on exec failing. Fix ancient, but unlikely, bug
- Better error messages on fork() & execl() failure
- Try to not reframe a buffer on redisplay. Ie display, display other, return to buffer. Keep same top line
- Fix bug accessing character beyond LINE end
- Fix bugs related to renew-buffer & pipe-command - not synching window to buffer 100%
- When setting user variable - check that the variable name is legal
- Fix bug with -ve counts: now show as -ve, don't reset to 1 when go -ve
* Wed Oct 17 2018 Alain Williams <addw@phcomp.co.uk>
- Bump to get Version 4.11ph - which comes from the Version: line in this file.
* Tue Oct 16 2018 Alain Williams <addw@phcomp.co.uk>
- Brief changes below, see Fixes
- Documentation greatly improved: almost full description; introduction to Emacs use; notes for other Emacs users
- New command find-current-tag, bind to M-T
- New command newline-and-maybe-indent, bind to C-J rather than newline-and-indent
- New commands: forget-file-name split-horizontally-and-find-file
- New command: append-file
- Add window option: POSITION-SHOW - show current column & line numbers in mode line.
- Implement compile time option of handling max Unicode/UTF-8 character > U+10FFFF - ie outside what the standard specifies.
- Accept Noncharacters in input files (http://www.unicode.org/versions/corrigendum9.html).
- Fix warnings shown by -Wconversion & similar - basically many more casts + tweaks on variable declarations
- A non current window with the hex option displays as highlight all the nybbles in a character
- Fix several small bugs all over.
- Enhancements in display, keyboard read & terminfo
- describe-key will now say if a character is self-inserting rather than unbound
- README file now built out of introduction & docs/MicroEmacsForEmacsUsers. Version # is substituted in.
- Licensing chatter in file LICENSE.
- Add to tracked commands: move-window-down & move-window-up
- Rework code that decodes input keys (eg C-a) to work with Unicode chars >= U+80 & other fixes
- Window structure: Now able to cope with terminal > 255 lines -- one will be along some time!
- Apropos command descriptions improved
- Rework kill/yank. Use LINEs in new [Kill Buffer]. Chars deleted now yank in the right order.
  Multiple, consequential delete/kills now put their deleted text into the same yank item.
- various commands now only mark the buffer as changed if they actually changed something, eg entab-line
- New var $french-spacing: fill-paragraph inserts 1 space at sentence end if this is TRUE (else 2)
- fill-paragraph rewritten. sentence end is now any of '.?!' - was just '.'
- Rework wrap-word: understand INDENT mode & if first word on the line go to end of word, not end of line
- 'set' command: global user variables will also be expanded by tabbing through.
- newline-and-indent now also copies leading non alpha characters
- fence movement (eg goto-matching-fence & C/PHP close bracket). Also marches <>. # introduces a comment. Many fixes
- Message line: now display, like other buffers. Won't overflow long lines, can have UTF-8 characters. Deep changes.
- Use 'ls' not 'echo' for file name expansion - now works with file names that contain spaces. Remove length limitation & limit on how many files
- Operating system (shell) commands: improve how prompt works when it is done.
- Implement numeric marks - automatically set by some commands, return to with return-alpha-markreturn-alpha-mark.
- Keyboard macros: now stored in a LINE, remove length limitation.
- Fix bug where not all modified files were flagged as such following C-z suspend, also done after shell command.
- Implement colours in message line messages
- Fix the bug that reset file permissions to read-only when they had been changed elsewhere
- execute-program fixed to prompt so that shell output can be read.
- Implement environment variables: $time-secs & $pid
- Change mode name TABEXPAND to TABEXP
- Note that MicroEmacs had a crypt bugfix in 2-dec-88 from Microemacs 5. Available as compile time option. This needs revisiting.
- 'Open/Writing' message now before the open, make same length as Sync message - optimise screen update
- Crypt passwords now echoed as **** and not visible in a history buffer
- insert-unicode now accepts #decimal - so can enter code points in decimal - helps with HTML entities, also ooctal
- On a crash the names of the last few commands run are printed, with their count & return status
- line-wrap (& WRAP mode) & in INDENT mode: rework line wrapping to avoid putting non joined alpha at line start
- Non regular files on command line have their types are shown by list-buffers
- Interactive search history now works in a sane way
- query-replace-string: add P option - prompt for different replacement string
- Rework search case independence (more needed). Now regardless of EXACT mode search is caseful if pattern as an UPPER case char
- Search: a space will match a space or a tab unless mode EXACT
- add-mode/add-window-option: Show current modes/options if ? is entered, ?? shows global/default ones
- A better way of getting line numbers for input: C-x . and C-x $
- RENEW mode: check every $renew-timeout secs and re-read the file if it has changed
- Prefix handling (M- & C-x & C-u) improved: uses keyboard timeout, not nasty signal()/alarm()
- If give +ve count when setting mode RENEW, it assumes that the file is appended to - only reads new lines
- Implement -M command line option
- U+1F now displays as ^~ or reverse ~ -- was ^_ or rev _ -- but the _ is hard to see, similar U+9F
- Pre-release of new regular expressions
* Thu May 18 2017 Alain Williams <addw@phcomp.co.uk>
- Version 4.9ph.
- Reverse the change where $environment variables had '-' replaced by '_'
- Ported to FreeBSD & Minix
- Control now shown as C- instead of ^ (eg C-x). ^ still accepted in bindings.
- Create commands move-to-window-down & relations, default bind to S-down-arrow, etc.
- Bindings accept CIRC for circumflex, SPC for space, DEL for delete, ESC for escape.
- Bindings: more prefixes, S- is shift, also ALT, eg S-home is Shift and Home; ^left-arrow, S-left-arrow, Alt-left-arrow, Alt-S-left-arrow
- Recognise things like Shift-arrow-key from terminfo
- Terminal output/redraw optimisations
  Fixes problem on laptop with glidepad where this could be accidentally typed.
- Implement command line option --list-keys
- get-and-goto-character now puts up prompt for the scan character
- Tweak apropos description text for get-and-goto-character & goto-character
- $build_date environment constant, when Emacs was compiled.
- Remove binding ^xr (reverse-incremental-search). Plan to use for new register commands
- Changes to the way that old documentation is packaged.
* Wed Feb 01 2017 Alain Williams <addw@phcomp.co.uk>
- Tiny fixes to make it build via RPMs again
- Fix bug in termcap initialisation.
* Tue Jan 31 2017 Alain Williams <addw@phcomp.co.uk>
- Wrap up many changes. Version 4.7ph.
- Main issue outstanding is no UTF-8 in command line
- In docs/: EditorFunctions intro tutorial at start, document all commands, work in progress; emacs.hlp updated
- Fixes file contains list of changes, summary of Hacks
- list-buffers (^x^b) shows much more information; not just about buffers, also provides explanations
- UTF-8: can fix encoding errors, many small errors fixed,
- Window options: COLOUR-TAB, META-ATTRIB, NUMBER (lines), SHOW-LINE-END
- Add add-window-option and delete-window-option (bound to M-o and M-^o)
- TRACK is now a window option, not a buffer mode.
- DOS mode - \r\n line endings
- TAB can be used as well as space to expand things like buffer names & Emacs commands
- Colours for cursor in non current window
- Rewrite filter-buffer to use pipes under Unix, not temp files. filter-region command written
- Buffer refresh from file: "." and marks are replaced where they were
- Optimise screen updates: use shorter cursor movement sequences; hardware line scroll; draw important lines first;
  look at baud rate & -S command line option
- Page up/down: preserve target column; stop on last page; up/downs better bring back to same line
- Add command description to describe-key and appropos/describe-bindings
- Bindings case sensitive, eg: M-c is distinct from M-C. Existing bindings to lower case version
- New trim-buffer command, leaves the buffer containing that number of lines
- commands: move-window-left & move-window-right; scroll text in a window left/right
  window-move-top window-move-bottom window-move-left window-move-right
- Startup files, now also look for emacs.rc in /etc/ amongst other places (as well as .emacsrc)
- Macro functions as per me 5.0: call gtc isnum reverse trim; also ~l ~e ~"
- Add recognition of all termcap function keys: FN13 - FN63
- New environment varaibles: $markcol, $markline, $utf8_replacement, $utf8_errno, $utf8_errlen, $curwind $numwind $fcol $lastmesg $od $time, $window_move_margin $page_context $autodos $term $filter_can_intr
  All '-' in env vars changed to '_'  
- Unix command line options: new --version; + (line #) can be -ve (count from file end), can specify column;
  --list-commands prints all Emacs commands;
  -N command line option: windows to have line numbers on startup; -S (slow terminals)
- Command prompts have history: different for different types of prompt
- Default bindings changed to be more consistent with use elsewhere for: ^B (backward-character), ^H (delete-previous-character)
- Bind toggle-insert-mode to insert key 
* Sun Jan 31 2016 Alain Williams <addw@phcomp.co.uk>
- Rework setting of envvars to apply range checks & a couple of similar
- Fix problems with display and movement related to characters of width != 1
- Initial message says how to get help
- Help file installed where Emacs can find it
- Fix exporting VERSION and DOCDIR to sub makefiles
- Invent $colourise-tab variable. When set tabs show in cyan
- Take $progname from command line rather than an in built constant
- Use POSIX types rather than long.
* Thu Jan 21 2016 Alain Williams <addw@phcomp.co.uk>
- Terminal reset too quickly after spawn() - so that command output could not be seen.
- If started without a file argument - the screen was not properly painted, no background colour.
- windowManipulate() now understands ^z (suspend)
- Documentation directory can be provided for finding emacs.hlp
- Tidy up impossible bindings -- M-Q
- Provide nice naming of things like arrow and function keys
- Fix crash when given command: M-ArrowKey
- Fix use of keytab[] was using size 256 not NBINDS
* Mon Jan 18 2016 Alain Williams <addw@phcomp.co.uk>
- Version change to 4.3ph
- Bugs fixed: background colour preserved on return from suspend.
- Not crash when window size shrunk too much or changed when a shell command is run.
- When a buffer is marked unchanged, remove the flag in the mode line in ALL windows in which it is displayed.
- Occasionally screen update left a rogue position badly displayed.
- Reverse the action of $catchsuspend - default is inverted so that the default action is the same.
- It was wrong in the first place; it now matches what the name suggests
- Left and right borders of current window now highlit with |.
- New bindable command window-menu: Allow windows to be split horis/vertically, windows to be joined,
- move edges, change buffer, kill/close.
- Change binding of ^Xw to window-menu.
* Fri Jun 12 2015 Alain Williams <addw@phcomp.co.uk>
- Version change to 4.002ph
- Major changes, most important is UTF-8/Unicode mode: able to display & enter them
- F5 F6 (cycle-foregound-colour cycle-backgound-colour)
- New vars $bg-colour $fg-colour - initial colours
- Meta & control characters shown as character underline/standout, set $meta-attrib for old behaviour
- Position of mark shown in red
- Last char of a over long line shown in yellow; go to it & whole screen scrolls not just one line
- list-buffers now shows variables
- Not catch signal 22 (SIGTTOU)
- update README
* Mon Feb 20 2012 Alain Williams <addw@phcomp.co.uk>
- Can find list of named marks with ^XA? Clean up named marks code
* Mon Feb 20 2012 Alain Williams <addw@phcomp.co.uk>
- On paragraph fill (M-Q) honour indent if the mode is set
- Expansions to happen in special buffers - in many cases
- Fix bug on -m command line option
- Tab in file name input does some expansion
- Fix crashes when ^X^A ^ ^XA (place/return alpha mark) are invoked
* Wed Jan 18 2012 Alain Williams <addw@phcomp.co.uk>
- Fix crash on deleting a window on a buffer displayed in another window
* Wed Jan 18 2012 Alain Williams <addw@phcomp.co.uk>
- Fix bug where deleting a visible buffer sometimes caused a crash
- Fix bug where keypad mode not always properly set after shell out/sub-command
- Fix search/replace bug where subects could be missed when too close together
- Fix bug where only the first window change size was actioned
- Stop crash in buffer refresh when more than one buffer is displayed. Also return to the current line if possible
* Fri Jan 13 2012 Alain Williams <addw@phcomp.co.uk>
- Fix buffer over run.
* Tue Dec 30 2008 Alain Williams <addw@phcomp.co.uk>
- First time as an RPM
